//
//  GWt_abstractElementFactory.h
//  PSPA
//
//  Created by Garnier Laurent on 28/01/13.
//  Copyright (c) 2013 Garnier Laurent. All rights reserved.
//

#ifndef PSPA_GWt_abstractElementFactory_h
#define PSPA_GWt_abstractElementFactory_h

#include "GWt_rfgun.h"
#include "GWt_drift.h"
#include "GWt_cell.h"
#include "GWt_bend.h"
#include "GWt_soleno.h"
#include "GWt_beam.h"
#include "GWt_fit.h"
#include "GWt_snapshot.h"
#include "GWt_mpole.h"

using namespace Wt;
using namespace std;

class GWt_abstractElementFactory 
{

 public :
  
  GWt_abstractElementFactory();
  ~GWt_abstractElementFactory();
  
  /* Enable the rfgun element creation
   */
  inline void setRFGunEnable(bool a) { isRFGunElementEnable = a;};
  
  /* Enable the drift element creation
   */
  inline void setDriftEnable(bool a) { isDriftElementEnable = a;};
  
  /* Enable the cell element creation
   */
  inline void setCellEnable(bool a) { isCellElementEnable = a;};
  
  /* Enable the bend element creation
   */
  inline void setBendEnable(bool a) { isBendElementEnable = a;};
  
  /* Enable the soleno element creation
   */
  inline void setSolenoEnable(bool a) { isSolenoElementEnable = a;};
  
  /* Enable the beam element creation
   */
  inline void setBeamEnable(bool a) { isBeamElementEnable = a;};
  
  /* Enable the fit element creation
   */
  inline void setFitEnable(bool a) { isFitElementEnable = a;};
  
  /* Enable the snapshot element creation
   */
  inline void setSnapshotEnable(bool a) { isSnapshotElementEnable = a;};
  
  /* Enable the mpole element creation
   */
  inline void setMPoleEnable(bool a) { isMPoleElementEnable = a;};
  
  GWt_drift* createDriftElement();
  GWt_rfgun* createRFGunElement();
  GWt_cell* createCellElement();
  GWt_bend* createBendElement();
  GWt_soleno* createSolenoElement();
  GWt_beam* createBeamElement();
  GWt_fit* createFitElement();
  GWt_snapshot* createSnapshotElement();
  GWt_mpole* createMPoleElement();
  vector <GWt_abstractElement*> getAllElements();
  
 private :

  bool isDriftElementEnable;
  bool isRFGunElementEnable;
  bool isCellElementEnable;
  bool isBendElementEnable;
  bool isSolenoElementEnable;
  bool isBeamElementEnable;
  bool isFitElementEnable;
  bool isSnapshotElementEnable;
  bool isMPoleElementEnable;
};
#endif
